/****************************************************************************
** Meta object code from reading C++ file 'storagedebugger.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/akonadi-1.13.0/server/src/storage/storagedebugger.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'storagedebugger.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Akonadi__Server__StorageDebugger[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: signature, parameters, type, tag, flags
      91,   34,   33,   33, 0x05,

       0        // eod
};

static const char qt_meta_stringdata_Akonadi__Server__StorageDebugger[] = {
    "Akonadi::Server::StorageDebugger\0\0"
    "sequence,duration,query,values,resultsCount,result,error\0"
    "queryExecuted(double,uint,QString,QMap<QString,QVariant>,int,QList<QLi"
    "st<QVariant> >,QString)\0"
};

void Akonadi::Server::StorageDebugger::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        StorageDebugger *_t = static_cast<StorageDebugger *>(_o);
        switch (_id) {
        case 0: _t->queryExecuted((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QMap<QString,QVariant>(*)>(_a[4])),(*reinterpret_cast< int(*)>(_a[5])),(*reinterpret_cast< const QList<QList<QVariant> >(*)>(_a[6])),(*reinterpret_cast< const QString(*)>(_a[7]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Akonadi::Server::StorageDebugger::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Akonadi::Server::StorageDebugger::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_Akonadi__Server__StorageDebugger,
      qt_meta_data_Akonadi__Server__StorageDebugger, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Akonadi::Server::StorageDebugger::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Akonadi::Server::StorageDebugger::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Akonadi::Server::StorageDebugger::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Akonadi__Server__StorageDebugger))
        return static_cast<void*>(const_cast< StorageDebugger*>(this));
    return QObject::qt_metacast(_clname);
}

int Akonadi::Server::StorageDebugger::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Akonadi::Server::StorageDebugger::queryExecuted(double _t1, uint _t2, const QString & _t3, const QMap<QString,QVariant> & _t4, int _t5, const QList<QList<QVariant> > & _t6, const QString & _t7)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)), const_cast<void*>(reinterpret_cast<const void*>(&_t4)), const_cast<void*>(reinterpret_cast<const void*>(&_t5)), const_cast<void*>(reinterpret_cast<const void*>(&_t6)), const_cast<void*>(reinterpret_cast<const void*>(&_t7)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
