/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -p resource_manager /usr/src/ports/akonadi/akonadi-1.13.0-6.x86_64/src/akonadi-1.13.0/interfaces/org.freedesktop.Akonadi.ResourceManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef RESOURCE_MANAGER_H_1515514800
#define RESOURCE_MANAGER_H_1515514800

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.freedesktop.Akonadi.ResourceManager
 */
class OrgFreedesktopAkonadiResourceManagerInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.Akonadi.ResourceManager"; }

public:
    OrgFreedesktopAkonadiResourceManagerInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~OrgFreedesktopAkonadiResourceManagerInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> addResourceInstance(const QString &identifier, const QStringList &capabilities)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(identifier) << QVariant::fromValue(capabilities);
        return asyncCallWithArgumentList(QLatin1String("addResourceInstance"), argumentList);
    }

    inline QDBusPendingReply<> removeResourceInstance(const QString &identifier)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(identifier);
        return asyncCallWithArgumentList(QLatin1String("removeResourceInstance"), argumentList);
    }

    inline QDBusPendingReply<QStringList> resourceInstances()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("resourceInstances"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace freedesktop {
    namespace Akonadi {
      typedef ::OrgFreedesktopAkonadiResourceManagerInterface ResourceManager;
    }
  }
}
#endif
