/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -p controlmanagerinterface /usr/src/ports/akonadi/akonadi-1.13.0-6.x86_64/src/akonadi-1.13.0/interfaces/org.freedesktop.Akonadi.ControlManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef CONTROLMANAGERINTERFACE_H_1515514773
#define CONTROLMANAGERINTERFACE_H_1515514773

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.freedesktop.Akonadi.ControlManager
 */
class OrgFreedesktopAkonadiControlManagerInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.Akonadi.ControlManager"; }

public:
    OrgFreedesktopAkonadiControlManagerInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~OrgFreedesktopAkonadiControlManagerInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> shutdown()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("shutdown"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace freedesktop {
    namespace Akonadi {
      typedef ::OrgFreedesktopAkonadiControlManagerInterface ControlManager;
    }
  }
}
#endif
