/****************************************************************************
** Meta object code from reading C++ file 'notificationsource.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/akonadi-1.13.0/server/src/notificationsource.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'notificationsource.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Akonadi__Server__NotificationSource[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
      29,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // classinfo: key, value
      79,   36,

 // signals: signature, parameters, type, tag, flags
     101,   96,   95,   95, 0x45,
     144,   96,   95,   95, 0x45,
     191,   96,   95,   95, 0x45,
     238,   95,   95,   95, 0x45,
     268,   95,   95,   95, 0x45,
     292,   95,   95,   95, 0x45,
     315,   95,   95,   95, 0x45,
     343,   95,   95,   95, 0x45,
     371,   95,   95,   95, 0x45,
     395,   95,   95,   95, 0x45,
     420,   95,   95,   95, 0x45,

 // slots: signature, parameters, type, tag, flags
     444,   95,   95,   95, 0x4a,
     471,  458,   95,   95, 0x4a,
     531,   95,  511,   95, 0x4a,
     554,  458,   95,   95, 0x4a,
     588,   95,  511,   95, 0x4a,
     605,  458,   95,   95, 0x4a,
     638,   95,  511,   95, 0x4a,
     673,  654,   95,   95, 0x4a,
     731,   95,  711,   95, 0x4a,
     771,  752,   95,   95, 0x4a,
     818,   95,  806,   95, 0x4a,
     852,  839,   95,   95, 0x4a,
     879,   95,  874,   95, 0x4a,
     914,  896,   95,   95, 0x4a,
     949,   95,  711,   95, 0x4a,
     982,  967,   95,   95, 0x4a,
    1070,   95, 1033,   95, 0x4a,
    1099, 1087,   95,   95, 0x08,

       0        // eod
};

static const char qt_meta_stringdata_Akonadi__Server__NotificationSource[] = {
    "Akonadi::Server::NotificationSource\0"
    "org.freedesktop.Akonadi.NotificationSource\0"
    "D-Bus Interface\0\0msgs\0"
    "notify(Akonadi::NotificationMessage::List)\0"
    "notifyV2(Akonadi::NotificationMessageV2::List)\0"
    "notifyV3(Akonadi::NotificationMessageV3::List)\0"
    "monitoredCollectionsChanged()\0"
    "monitoredItemsChanged()\0monitoredTagsChanged()\0"
    "monitoredResourcesChanged()\0"
    "monitoredMimeTypesChanged()\0"
    "isAllMonitoredChanged()\0"
    "ignoredSessionsChanged()\0"
    "monitoredTypesChanged()\0unsubscribe()\0"
    "id,monitored\0setMonitoredCollection(Entity::Id,bool)\0"
    "QVector<Entity::Id>\0monitoredCollections()\0"
    "setMonitoredItem(Entity::Id,bool)\0"
    "monitoredItems()\0setMonitoredTag(Entity::Id,bool)\0"
    "monitoredTags()\0resource,monitored\0"
    "setMonitoredResource(QByteArray,bool)\0"
    "QVector<QByteArray>\0monitoredResources()\0"
    "mimeType,monitored\0"
    "setMonitoredMimeType(QString,bool)\0"
    "QStringList\0monitoredMimeTypes()\0"
    "allMonitored\0setAllMonitored(bool)\0"
    "bool\0isAllMonitored()\0sessionId,ignored\0"
    "setIgnoredSession(QByteArray,bool)\0"
    "ignoredSessions()\0type,monitored\0"
    "setMonitoredType(NotificationMessageV2::Type,bool)\0"
    "QVector<NotificationMessageV2::Type>\0"
    "monitoredTypes()\0serviceName\0"
    "serviceUnregistered(QString)\0"
};

void Akonadi::Server::NotificationSource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        NotificationSource *_t = static_cast<NotificationSource *>(_o);
        switch (_id) {
        case 0: _t->notify((*reinterpret_cast< const Akonadi::NotificationMessage::List(*)>(_a[1]))); break;
        case 1: _t->notifyV2((*reinterpret_cast< const Akonadi::NotificationMessageV2::List(*)>(_a[1]))); break;
        case 2: _t->notifyV3((*reinterpret_cast< const Akonadi::NotificationMessageV3::List(*)>(_a[1]))); break;
        case 3: _t->monitoredCollectionsChanged(); break;
        case 4: _t->monitoredItemsChanged(); break;
        case 5: _t->monitoredTagsChanged(); break;
        case 6: _t->monitoredResourcesChanged(); break;
        case 7: _t->monitoredMimeTypesChanged(); break;
        case 8: _t->isAllMonitoredChanged(); break;
        case 9: _t->ignoredSessionsChanged(); break;
        case 10: _t->monitoredTypesChanged(); break;
        case 11: _t->unsubscribe(); break;
        case 12: _t->setMonitoredCollection((*reinterpret_cast< Entity::Id(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 13: { QVector<Entity::Id> _r = _t->monitoredCollections();
            if (_a[0]) *reinterpret_cast< QVector<Entity::Id>*>(_a[0]) = _r; }  break;
        case 14: _t->setMonitoredItem((*reinterpret_cast< Entity::Id(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 15: { QVector<Entity::Id> _r = _t->monitoredItems();
            if (_a[0]) *reinterpret_cast< QVector<Entity::Id>*>(_a[0]) = _r; }  break;
        case 16: _t->setMonitoredTag((*reinterpret_cast< Entity::Id(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 17: { QVector<Entity::Id> _r = _t->monitoredTags();
            if (_a[0]) *reinterpret_cast< QVector<Entity::Id>*>(_a[0]) = _r; }  break;
        case 18: _t->setMonitoredResource((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 19: { QVector<QByteArray> _r = _t->monitoredResources();
            if (_a[0]) *reinterpret_cast< QVector<QByteArray>*>(_a[0]) = _r; }  break;
        case 20: _t->setMonitoredMimeType((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 21: { QStringList _r = _t->monitoredMimeTypes();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = _r; }  break;
        case 22: _t->setAllMonitored((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: { bool _r = _t->isAllMonitored();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 24: _t->setIgnoredSession((*reinterpret_cast< const QByteArray(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 25: { QVector<QByteArray> _r = _t->ignoredSessions();
            if (_a[0]) *reinterpret_cast< QVector<QByteArray>*>(_a[0]) = _r; }  break;
        case 26: _t->setMonitoredType((*reinterpret_cast< NotificationMessageV2::Type(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 27: { QVector<NotificationMessageV2::Type> _r = _t->monitoredTypes();
            if (_a[0]) *reinterpret_cast< QVector<NotificationMessageV2::Type>*>(_a[0]) = _r; }  break;
        case 28: _t->serviceUnregistered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Akonadi::Server::NotificationSource::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Akonadi::Server::NotificationSource::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_Akonadi__Server__NotificationSource,
      qt_meta_data_Akonadi__Server__NotificationSource, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Akonadi::Server::NotificationSource::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Akonadi::Server::NotificationSource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Akonadi::Server::NotificationSource::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Akonadi__Server__NotificationSource))
        return static_cast<void*>(const_cast< NotificationSource*>(this));
    return QObject::qt_metacast(_clname);
}

int Akonadi::Server::NotificationSource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    return _id;
}

// SIGNAL 0
void Akonadi::Server::NotificationSource::notify(const Akonadi::NotificationMessage::List & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Akonadi::Server::NotificationSource::notifyV2(const Akonadi::NotificationMessageV2::List & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Akonadi::Server::NotificationSource::notifyV3(const Akonadi::NotificationMessageV3::List & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Akonadi::Server::NotificationSource::monitoredCollectionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, 0);
}

// SIGNAL 4
void Akonadi::Server::NotificationSource::monitoredItemsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, 0);
}

// SIGNAL 5
void Akonadi::Server::NotificationSource::monitoredTagsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, 0);
}

// SIGNAL 6
void Akonadi::Server::NotificationSource::monitoredResourcesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, 0);
}

// SIGNAL 7
void Akonadi::Server::NotificationSource::monitoredMimeTypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, 0);
}

// SIGNAL 8
void Akonadi::Server::NotificationSource::isAllMonitoredChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, 0);
}

// SIGNAL 9
void Akonadi::Server::NotificationSource::ignoredSessionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, 0);
}

// SIGNAL 10
void Akonadi::Server::NotificationSource::monitoredTypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, 0);
}
QT_END_MOC_NAMESPACE
