/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -p controlmanagerinterface /usr/src/ports/akonadi/akonadi-1.13.0-6.x86_64/src/akonadi-1.13.0/interfaces/org.freedesktop.Akonadi.ControlManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "controlmanagerinterface.h"

/*
 * Implementation of interface class OrgFreedesktopAkonadiControlManagerInterface
 */

OrgFreedesktopAkonadiControlManagerInterface::OrgFreedesktopAkonadiControlManagerInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgFreedesktopAkonadiControlManagerInterface::~OrgFreedesktopAkonadiControlManagerInterface()
{
}


#include "controlmanagerinterface.moc"
