/****************************************************************************
** Meta object code from reading C++ file 'storagejanitor.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/akonadi-1.13.0/server/src/storagejanitor.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'storagejanitor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Akonadi__Server__StorageJanitorThread[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

static const char qt_meta_stringdata_Akonadi__Server__StorageJanitorThread[] = {
    "Akonadi::Server::StorageJanitorThread\0"
};

void Akonadi::Server::StorageJanitorThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    Q_UNUSED(_o);
    Q_UNUSED(_id);
    Q_UNUSED(_c);
    Q_UNUSED(_a);
}

const QMetaObjectExtraData Akonadi::Server::StorageJanitorThread::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Akonadi::Server::StorageJanitorThread::staticMetaObject = {
    { &QThread::staticMetaObject, qt_meta_stringdata_Akonadi__Server__StorageJanitorThread,
      qt_meta_data_Akonadi__Server__StorageJanitorThread, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Akonadi::Server::StorageJanitorThread::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Akonadi::Server::StorageJanitorThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Akonadi::Server::StorageJanitorThread::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Akonadi__Server__StorageJanitorThread))
        return static_cast<void*>(const_cast< StorageJanitorThread*>(this));
    return QThread::qt_metacast(_clname);
}

int Akonadi::Server::StorageJanitorThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    return _id;
}
static const uint qt_meta_data_Akonadi__Server__StorageJanitor[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
       3,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // classinfo: key, value
      64,   32,

 // signals: signature, parameters, type, tag, flags
      85,   81,   80,   80, 0x45,

 // slots: signature, parameters, type, tag, flags
     116,   80,   80,  106, 0x4a,
     124,   80,   80,  106, 0x4a,

       0        // eod
};

static const char qt_meta_stringdata_Akonadi__Server__StorageJanitor[] = {
    "Akonadi::Server::StorageJanitor\0"
    "org.freedesktop.Akonadi.Janitor\0"
    "D-Bus Interface\0\0msg\0information(QString)\0"
    "Q_NOREPLY\0check()\0vacuum()\0"
};

void Akonadi::Server::StorageJanitor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        StorageJanitor *_t = static_cast<StorageJanitor *>(_o);
        switch (_id) {
        case 0: _t->information((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->check(); break;
        case 2: _t->vacuum(); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Akonadi::Server::StorageJanitor::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Akonadi::Server::StorageJanitor::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_Akonadi__Server__StorageJanitor,
      qt_meta_data_Akonadi__Server__StorageJanitor, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Akonadi::Server::StorageJanitor::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Akonadi::Server::StorageJanitor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Akonadi::Server::StorageJanitor::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Akonadi__Server__StorageJanitor))
        return static_cast<void*>(const_cast< StorageJanitor*>(this));
    return QObject::qt_metacast(_clname);
}

int Akonadi::Server::StorageJanitor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void Akonadi::Server::StorageJanitor::information(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
