/****************************************************************************
** Meta object code from reading C++ file 'agentmanager.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/akonadi-1.13.0/server/control/agentmanager.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'agentmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_AgentManager[] = {

 // content:
       6,       // revision
       0,       // classname
       1,   14, // classinfo
      46,   16, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      11,       // signalCount

 // classinfo: key, value
      50,   13,

 // signals: signature, parameters, type, tag, flags
      77,   67,   66,   66, 0x05,
     101,   67,   66,   66, 0x05,
     143,  127,   66,   66, 0x05,
     171,  127,   66,   66, 0x05,
     232,  201,   66,   66, 0x05,
     303,  280,   66,   66, 0x05,
     392,  359,   66,   66, 0x05,
     467,  443,   66,   66, 0x05,
     505,  443,   66,   66, 0x05,
     562,  541,   66,   66, 0x05,
     626,  604,   66,   66, 0x05,

 // slots: signature, parameters, type, tag, flags
     679,   66,  667,   66, 0x0a,
     720,  700,  692,   66, 0x0a,
     758,  747,  692,   66, 0x2a,
     777,  700,  692,   66, 0x0a,
     807,  747,  692,   66, 0x2a,
     829,  747,  692,   66, 0x0a,
     848,  747,  667,   66, 0x0a,
     872,  747,  667,   66, 0x0a,
     911,  747,  899,   66, 0x0a,
     942,  747,  692,   66, 0x0a,
     971,  747,   66,   66, 0x0a,
    1000,  747,  692,   66, 0x0a,
    1027,   66,  667,   66, 0x0a,
    1048,  747, 1044,   66, 0x0a,
    1077,  747,  692,   66, 0x0a,
    1118,  747, 1113,   66, 0x0a,
    1149,  747,  692,   66, 0x0a,
    1203, 1187,   66,   66, 0x0a,
    1241,  700,  692,   66, 0x0a,
    1276,  747,  692,   66, 0x2a,
    1323, 1303,   66,   66, 0x0a,
    1365,  747,   66,   66, 0x0a,
    1399,  747,   66,   66, 0x0a,
    1469, 1447,   66,   66, 0x0a,
    1552, 1520,   66,   66, 0x0a,
    1613,  747, 1608,   66, 0x0a,
    1659, 1642,   66,   66, 0x0a,
    1696,  747,   66,   66, 0x0a,
    1765, 1726,   66,   66, 0x0a,
    1818, 1799,   66,   66, 0x0a,
    1840,   66,   66,   66, 0x08,
    1883, 1860,   66,   66, 0x08,
    1937, 1928,   66,   66, 0x08,
    1962,   66,   66,   66, 0x08,
    1983,   66,   66,   66, 0x08,

       0        // eod
};

static const char qt_meta_stringdata_AgentManager[] = {
    "AgentManager\0org.freedesktop.Akonadi.AgentManager\0"
    "D-Bus Interface\0\0agentType\0"
    "agentTypeAdded(QString)\0"
    "agentTypeRemoved(QString)\0agentIdentifier\0"
    "agentInstanceAdded(QString)\0"
    "agentInstanceRemoved(QString)\0"
    "agentIdentifier,status,message\0"
    "agentInstanceStatusChanged(QString,int,QString)\0"
    "agentIdentifier,status\0"
    "agentInstanceAdvancedStatusChanged(QString,QVariantMap)\0"
    "agentIdentifier,progress,message\0"
    "agentInstanceProgressChanged(QString,uint,QString)\0"
    "agentIdentifier,message\0"
    "agentInstanceWarning(QString,QString)\0"
    "agentInstanceError(QString,QString)\0"
    "agentIdentifier,name\0"
    "agentInstanceNameChanged(QString,QString)\0"
    "agentIdentifier,state\0"
    "agentInstanceOnlineChanged(QString,bool)\0"
    "QStringList\0agentTypes()\0QString\0"
    "identifier,language\0agentName(QString,QString)\0"
    "identifier\0agentName(QString)\0"
    "agentComment(QString,QString)\0"
    "agentComment(QString)\0agentIcon(QString)\0"
    "agentMimeTypes(QString)\0"
    "agentCapabilities(QString)\0QVariantMap\0"
    "agentCustomProperties(QString)\0"
    "createAgentInstance(QString)\0"
    "removeAgentInstance(QString)\0"
    "agentInstanceType(QString)\0agentInstances()\0"
    "int\0agentInstanceStatus(QString)\0"
    "agentInstanceStatusMessage(QString)\0"
    "uint\0agentInstanceProgress(QString)\0"
    "agentInstanceProgressMessage(QString)\0"
    "identifier,name\0setAgentInstanceName(QString,QString)\0"
    "agentInstanceName(QString,QString)\0"
    "agentInstanceName(QString)\0"
    "identifier,windowId\0"
    "agentInstanceConfigure(QString,qlonglong)\0"
    "agentInstanceSynchronize(QString)\0"
    "agentInstanceSynchronizeCollectionTree(QString)\0"
    "identifier,collection\0"
    "agentInstanceSynchronizeCollection(QString,qint64)\0"
    "identifier,collection,recursive\0"
    "agentInstanceSynchronizeCollection(QString,qint64,bool)\0"
    "bool\0agentInstanceOnline(QString)\0"
    "identifier,state\0setAgentInstanceOnline(QString,bool)\0"
    "restartAgentInstance(QString)\0"
    "query,queryLanguage,resultCollectionId\0"
    "addSearch(QString,QString,qint64)\0"
    "resultCollectionId\0removeSearch(quint64)\0"
    "updatePluginInfos()\0name,oldOwner,newOwner\0"
    "serviceOwnerChanged(QString,QString,QString)\0"
    "fileName\0agentExeChanged(QString)\0"
    "agentServerFailure()\0serverFailure()\0"
};

void AgentManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        AgentManager *_t = static_cast<AgentManager *>(_o);
        switch (_id) {
        case 0: _t->agentTypeAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->agentTypeRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->agentInstanceAdded((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->agentInstanceRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->agentInstanceStatusChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 5: _t->agentInstanceAdvancedStatusChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QVariantMap(*)>(_a[2]))); break;
        case 6: _t->agentInstanceProgressChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 7: _t->agentInstanceWarning((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 8: _t->agentInstanceError((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 9: _t->agentInstanceNameChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 10: _t->agentInstanceOnlineChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 11: { QStringList _r = _t->agentTypes();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = _r; }  break;
        case 12: { QString _r = _t->agentName((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 13: { QString _r = _t->agentName((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 14: { QString _r = _t->agentComment((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 15: { QString _r = _t->agentComment((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 16: { QString _r = _t->agentIcon((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 17: { QStringList _r = _t->agentMimeTypes((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = _r; }  break;
        case 18: { QStringList _r = _t->agentCapabilities((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = _r; }  break;
        case 19: { QVariantMap _r = _t->agentCustomProperties((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QVariantMap*>(_a[0]) = _r; }  break;
        case 20: { QString _r = _t->createAgentInstance((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 21: _t->removeAgentInstance((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 22: { QString _r = _t->agentInstanceType((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 23: { QStringList _r = _t->agentInstances();
            if (_a[0]) *reinterpret_cast< QStringList*>(_a[0]) = _r; }  break;
        case 24: { int _r = _t->agentInstanceStatus((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = _r; }  break;
        case 25: { QString _r = _t->agentInstanceStatusMessage((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 26: { uint _r = _t->agentInstanceProgress((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = _r; }  break;
        case 27: { QString _r = _t->agentInstanceProgressMessage((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 28: _t->setAgentInstanceName((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 29: { QString _r = _t->agentInstanceName((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 30: { QString _r = _t->agentInstanceName((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 31: _t->agentInstanceConfigure((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qlonglong(*)>(_a[2]))); break;
        case 32: _t->agentInstanceSynchronize((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 33: _t->agentInstanceSynchronizeCollectionTree((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 34: _t->agentInstanceSynchronizeCollection((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 35: _t->agentInstanceSynchronizeCollection((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 36: { bool _r = _t->agentInstanceOnline((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 37: _t->setAgentInstanceOnline((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 38: _t->restartAgentInstance((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 39: _t->addSearch((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< qint64(*)>(_a[3]))); break;
        case 40: _t->removeSearch((*reinterpret_cast< quint64(*)>(_a[1]))); break;
        case 41: _t->updatePluginInfos(); break;
        case 42: _t->serviceOwnerChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 43: _t->agentExeChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 44: _t->agentServerFailure(); break;
        case 45: _t->serverFailure(); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData AgentManager::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject AgentManager::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_AgentManager,
      qt_meta_data_AgentManager, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &AgentManager::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *AgentManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *AgentManager::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_AgentManager))
        return static_cast<void*>(const_cast< AgentManager*>(this));
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(const_cast< AgentManager*>(this));
    return QObject::qt_metacast(_clname);
}

int AgentManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 46)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 46;
    }
    return _id;
}

// SIGNAL 0
void AgentManager::agentTypeAdded(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void AgentManager::agentTypeRemoved(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void AgentManager::agentInstanceAdded(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void AgentManager::agentInstanceRemoved(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void AgentManager::agentInstanceStatusChanged(const QString & _t1, int _t2, const QString & _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void AgentManager::agentInstanceAdvancedStatusChanged(const QString & _t1, const QVariantMap & _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void AgentManager::agentInstanceProgressChanged(const QString & _t1, uint _t2, const QString & _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void AgentManager::agentInstanceWarning(const QString & _t1, const QString & _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void AgentManager::agentInstanceError(const QString & _t1, const QString & _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void AgentManager::agentInstanceNameChanged(const QString & _t1, const QString & _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void AgentManager::agentInstanceOnlineChanged(const QString & _t1, bool _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}
QT_END_MOC_NAMESPACE
