/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a notificationsourceadaptor -i notificationsource.h -l Akonadi::Server::NotificationSource /usr/src/ports/akonadi/akonadi-1.13.0-6.x86_64/src/akonadi-1.13.0/interfaces/org.freedesktop.Akonadi.NotificationSource.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "notificationsourceadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class NotificationSourceAdaptor
 */

NotificationSourceAdaptor::NotificationSourceAdaptor(Akonadi::Server::NotificationSource *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

NotificationSourceAdaptor::~NotificationSourceAdaptor()
{
    // destructor
}

QVector<QByteArray> NotificationSourceAdaptor::ignoredSessions()
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.ignoredSessions
    return parent()->ignoredSessions();
}

bool NotificationSourceAdaptor::isAllMonitored()
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.isAllMonitored
    return parent()->isAllMonitored();
}

QVector<qint64> NotificationSourceAdaptor::monitoredCollections()
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.monitoredCollections
    return parent()->monitoredCollections();
}

QVector<qint64> NotificationSourceAdaptor::monitoredItems()
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.monitoredItems
    return parent()->monitoredItems();
}

QStringList NotificationSourceAdaptor::monitoredMimeTypes()
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.monitoredMimeTypes
    return parent()->monitoredMimeTypes();
}

QVector<QByteArray> NotificationSourceAdaptor::monitoredResources()
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.monitoredResources
    return parent()->monitoredResources();
}

QVector<qint64> NotificationSourceAdaptor::monitoredTags()
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.monitoredTags
    return parent()->monitoredTags();
}

QVector<Akonadi::NotificationMessageV2::Type> NotificationSourceAdaptor::monitoredTypes()
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.monitoredTypes
    return parent()->monitoredTypes();
}

void NotificationSourceAdaptor::setAllMonitored(bool allMonitored)
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.setAllMonitored
    parent()->setAllMonitored(allMonitored);
}

void NotificationSourceAdaptor::setIgnoredSession(const QByteArray &session, bool ignored)
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.setIgnoredSession
    parent()->setIgnoredSession(session, ignored);
}

void NotificationSourceAdaptor::setMonitoredCollection(qlonglong collection, bool monitored)
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.setMonitoredCollection
    parent()->setMonitoredCollection(collection, monitored);
}

void NotificationSourceAdaptor::setMonitoredItem(qlonglong item, bool monitored)
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.setMonitoredItem
    parent()->setMonitoredItem(item, monitored);
}

void NotificationSourceAdaptor::setMonitoredMimeType(const QString &mimetype, bool monitored)
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.setMonitoredMimeType
    parent()->setMonitoredMimeType(mimetype, monitored);
}

void NotificationSourceAdaptor::setMonitoredResource(const QByteArray &resource, bool monitored)
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.setMonitoredResource
    parent()->setMonitoredResource(resource, monitored);
}

void NotificationSourceAdaptor::setMonitoredTag(qlonglong tag, bool monitored)
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.setMonitoredTag
    parent()->setMonitoredTag(tag, monitored);
}

void NotificationSourceAdaptor::setMonitoredType(Akonadi::NotificationMessageV2::Type type, bool monitored)
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.setMonitoredType
    parent()->setMonitoredType(type, monitored);
}

void NotificationSourceAdaptor::unsubscribe()
{
    // handle method call org.freedesktop.Akonadi.NotificationSource.unsubscribe
    parent()->unsubscribe();
}


#include "notificationsourceadaptor.moc"
