/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a notificationmanageradaptor -i notificationmanager.h -l Akonadi::Server::NotificationManager /usr/src/ports/akonadi/akonadi-1.13.0-6.x86_64/src/akonadi-1.13.0/interfaces/org.freedesktop.Akonadi.NotificationManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "notificationmanageradaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class NotificationManagerAdaptor
 */

NotificationManagerAdaptor::NotificationManagerAdaptor(Akonadi::Server::NotificationManager *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

NotificationManagerAdaptor::~NotificationManagerAdaptor()
{
    // destructor
}

QDBusObjectPath NotificationManagerAdaptor::subscribe(const QString &identifier)
{
    // handle method call org.freedesktop.Akonadi.NotificationManager.subscribe
    return parent()->subscribe(identifier);
}

QDBusObjectPath NotificationManagerAdaptor::subscribeV2(const QString &identifier, bool serverSideMonitor)
{
    // handle method call org.freedesktop.Akonadi.NotificationManager.subscribeV2
    return parent()->subscribeV2(identifier, serverSideMonitor);
}

QStringList NotificationManagerAdaptor::subscribers()
{
    // handle method call org.freedesktop.Akonadi.NotificationManager.subscribers
    return parent()->subscribers();
}

void NotificationManagerAdaptor::unsubscribe(const QString &identifier)
{
    // handle method call org.freedesktop.Akonadi.NotificationManager.unsubscribe
    parent()->unsubscribe(identifier);
}


#include "notificationmanageradaptor.moc"
