/****************************************************************************
** Meta object code from reading C++ file 'processcontrol.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/akonadi-1.13.0/server/control/processcontrol.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'processcontrol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Akonadi__ProcessControl[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: signature, parameters, type, tag, flags
      34,   25,   24,   24, 0x05,
      64,   24,   24,   24, 0x05,
      76,   24,   24,   24, 0x05,

 // slots: signature, parameters, type, tag, flags
      92,   24,   24,   24, 0x08,
     128,  126,   24,   24, 0x08,
     167,   24,   24,   24, 0x08,

       0        // eod
};

static const char qt_meta_stringdata_Akonadi__ProcessControl[] = {
    "Akonadi::ProcessControl\0\0errorMsg\0"
    "processErrorMessages(QString)\0restarted()\0"
    "unableToStart()\0slotError(QProcess::ProcessError)\0"
    ",\0slotFinished(int,QProcess::ExitStatus)\0"
    "resetCrashCount()\0"
};

void Akonadi::ProcessControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        ProcessControl *_t = static_cast<ProcessControl *>(_o);
        switch (_id) {
        case 0: _t->processErrorMessages((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->restarted(); break;
        case 2: _t->unableToStart(); break;
        case 3: _t->slotError((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 4: _t->slotFinished((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        case 5: _t->resetCrashCount(); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Akonadi::ProcessControl::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Akonadi::ProcessControl::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_Akonadi__ProcessControl,
      qt_meta_data_Akonadi__ProcessControl, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Akonadi::ProcessControl::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Akonadi::ProcessControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Akonadi::ProcessControl::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Akonadi__ProcessControl))
        return static_cast<void*>(const_cast< ProcessControl*>(this));
    return QObject::qt_metacast(_clname);
}

int Akonadi::ProcessControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Akonadi::ProcessControl::processErrorMessages(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Akonadi::ProcessControl::restarted()
{
    QMetaObject::activate(this, &staticMetaObject, 1, 0);
}

// SIGNAL 2
void Akonadi::ProcessControl::unableToStart()
{
    QMetaObject::activate(this, &staticMetaObject, 2, 0);
}
QT_END_MOC_NAMESPACE
