/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a debuginterfaceadaptor -i debuginterface.h -l Akonadi::Server::DebugInterface /usr/src/ports/akonadi/akonadi-1.13.0-6.x86_64/build/server/org.freedesktop.Akonadi.DebugInterface.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "debuginterfaceadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class DebugInterfaceAdaptor
 */

DebugInterfaceAdaptor::DebugInterfaceAdaptor(Akonadi::Server::DebugInterface *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

DebugInterfaceAdaptor::~DebugInterfaceAdaptor()
{
    // destructor
}

void DebugInterfaceAdaptor::setTracer(const QString &tracer)
{
    // handle method call org.freedesktop.Akonadi.DebugInterface.setTracer
    parent()->setTracer(tracer);
}

QString DebugInterfaceAdaptor::tracer()
{
    // handle method call org.freedesktop.Akonadi.DebugInterface.tracer
    return parent()->tracer();
}


#include "debuginterfaceadaptor.moc"
