/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -p preprocessorinterface /usr/src/ports/akonadi/akonadi-1.13.0-6.x86_64/src/akonadi-1.13.0/interfaces/org.freedesktop.Akonadi.Preprocessor.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef PREPROCESSORINTERFACE_H_1515514870
#define PREPROCESSORINTERFACE_H_1515514870

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.freedesktop.Akonadi.Preprocessor
 */
class OrgFreedesktopAkonadiPreprocessorInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.Akonadi.Preprocessor"; }

public:
    OrgFreedesktopAkonadiPreprocessorInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~OrgFreedesktopAkonadiPreprocessorInterface();

public Q_SLOTS: // METHODS
    inline Q_NOREPLY void beginProcessItem(qlonglong id, qlonglong collectionId, const QString &mimeType)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(id) << QVariant::fromValue(collectionId) << QVariant::fromValue(mimeType);
        callWithArgumentList(QDBus::NoBlock, QLatin1String("beginProcessItem"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void itemProcessed(qlonglong id);
};

namespace org {
  namespace freedesktop {
    namespace Akonadi {
      typedef ::OrgFreedesktopAkonadiPreprocessorInterface Preprocessor;
    }
  }
}
#endif
