/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -p resourceinterface /usr/src/ports/akonadi/akonadi-1.13.0-6.x86_64/src/akonadi-1.13.0/interfaces/org.freedesktop.Akonadi.Resource.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef RESOURCEINTERFACE_H_1515514869
#define RESOURCEINTERFACE_H_1515514869

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.freedesktop.Akonadi.Resource
 */
class OrgFreedesktopAkonadiResourceInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.Akonadi.Resource"; }

public:
    OrgFreedesktopAkonadiResourceInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = 0);

    ~OrgFreedesktopAkonadiResourceInterface();

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<QString> name()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QLatin1String("name"), argumentList);
    }

    inline QDBusPendingReply<bool> requestItemDelivery(qlonglong uid, const QString &remoteId, const QString &mimeType, const QStringList &parts)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(uid) << QVariant::fromValue(remoteId) << QVariant::fromValue(mimeType) << QVariant::fromValue(parts);
        return asyncCallWithArgumentList(QLatin1String("requestItemDelivery"), argumentList);
    }

    inline QDBusPendingReply<QString> requestItemDeliveryV2(qlonglong uid, const QString &remoteId, const QString &mimeType, const QStringList &parts)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(uid) << QVariant::fromValue(remoteId) << QVariant::fromValue(mimeType) << QVariant::fromValue(parts);
        return asyncCallWithArgumentList(QLatin1String("requestItemDeliveryV2"), argumentList);
    }

    inline QDBusPendingReply<> setName(const QString &name)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(name);
        return asyncCallWithArgumentList(QLatin1String("setName"), argumentList);
    }

    inline Q_NOREPLY void synchronize()
    {
        QList<QVariant> argumentList;
        callWithArgumentList(QDBus::NoBlock, QLatin1String("synchronize"), argumentList);
    }

    inline Q_NOREPLY void synchronizeCollection(qlonglong collectionId, bool recursive)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(collectionId) << QVariant::fromValue(recursive);
        callWithArgumentList(QDBus::NoBlock, QLatin1String("synchronizeCollection"), argumentList);
    }

    inline Q_NOREPLY void synchronizeCollection(qlonglong collectionId)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(collectionId);
        callWithArgumentList(QDBus::NoBlock, QLatin1String("synchronizeCollection"), argumentList);
    }

    inline Q_NOREPLY void synchronizeCollectionAttributes(qlonglong collectionId)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(collectionId);
        callWithArgumentList(QDBus::NoBlock, QLatin1String("synchronizeCollectionAttributes"), argumentList);
    }

    inline Q_NOREPLY void synchronizeCollectionTree()
    {
        QList<QVariant> argumentList;
        callWithArgumentList(QDBus::NoBlock, QLatin1String("synchronizeCollectionTree"), argumentList);
    }

Q_SIGNALS: // SIGNALS
    void attributesSynchronized(qlonglong collectionId);
    void collectionTreeSynchronized();
    void nameChanged(const QString &name);
    void synchronized();
};

namespace org {
  namespace freedesktop {
    namespace Akonadi {
      typedef ::OrgFreedesktopAkonadiResourceInterface Resource;
    }
  }
}
#endif
