/****************************************************************************
** Meta object code from reading C++ file 'tracer.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/akonadi-1.13.0/server/src/tracer.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tracer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Akonadi__Server__Tracer[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: signature, parameters, type, tag, flags
      40,   25,   24,   24, 0x0a,
      73,   25,   24,   24, 0x0a,
     104,   25,   24,   24, 0x0a,
     140,   25,   24,   24, 0x0a,
     192,  177,   24,   24, 0x0a,
     216,  177,   24,   24, 0x0a,
     262,  244,   24,   24, 0x0a,
     287,  244,   24,   24, 0x0a,
     310,  244,   24,   24, 0x0a,
     345,   24,  337,   24, 0x0a,
     366,  361,   24,   24, 0x0a,

       0        // eod
};

static const char qt_meta_stringdata_Akonadi__Server__Tracer[] = {
    "Akonadi::Server::Tracer\0\0identifier,msg\0"
    "beginConnection(QString,QString)\0"
    "endConnection(QString,QString)\0"
    "connectionInput(QString,QByteArray)\0"
    "connectionOutput(QString,QByteArray)\0"
    "signalName,msg\0signal(QString,QString)\0"
    "signal(const char*,QString)\0"
    "componentName,msg\0warning(QString,QString)\0"
    "error(QString,QString)\0"
    "error(const char*,QString)\0QString\0"
    "currentTracer()\0type\0activateTracer(QString)\0"
};

void Akonadi::Server::Tracer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        Tracer *_t = static_cast<Tracer *>(_o);
        switch (_id) {
        case 0: _t->beginConnection((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->endConnection((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 2: _t->connectionInput((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 3: _t->connectionOutput((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 4: _t->signal((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 5: _t->signal((*reinterpret_cast< const char*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->warning((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 7: _t->error((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 8: _t->error((*reinterpret_cast< const char*(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 9: { QString _r = _t->currentTracer();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = _r; }  break;
        case 10: _t->activateTracer((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Akonadi::Server::Tracer::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Akonadi::Server::Tracer::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_Akonadi__Server__Tracer,
      qt_meta_data_Akonadi__Server__Tracer, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Akonadi::Server::Tracer::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Akonadi::Server::Tracer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Akonadi::Server::Tracer::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Akonadi__Server__Tracer))
        return static_cast<void*>(const_cast< Tracer*>(this));
    if (!strcmp(_clname, "TracerInterface"))
        return static_cast< TracerInterface*>(const_cast< Tracer*>(this));
    return QObject::qt_metacast(_clname);
}

int Akonadi::Server::Tracer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}
QT_END_MOC_NAMESPACE
